@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2016 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -280,6 +280,7 @@ if (darcy_anisotropic_permeability.eq.0) then
     do iel = 1, ncel
       cpro_wgrec_s(iel) = cpro_permeability(iel)
     enddo
+    call synsca(cpro_wgrec_s)
   endif
 else if (darcy_anisotropic_permeability.eq.1) then
   allocate(weighf(2,nfac))
@@ -297,6 +298,7 @@ else if (darcy_anisotropic_permeability.eq.1) then
         cpro_wgrec_v(isou,iel) = cpro_permeability_6(isou,iel)
       enddo
     enddo
+    call syntis(cpro_wgrec_v)
   endif
 endif
 
@@ -367,16 +369,16 @@ else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrgrv &
   !==========
-( init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
-  iphydr , iwarnp ,                                                     &
-  epsrgp , climgp , extrap ,                                            &
-  rvoid  ,                                                              &
-  cvar_pr   ,                                                           &
-  coefa_p  , coefb_p  ,                                                 &
-  coefaf_p , coefbf_p ,                                                 &
-  viscf  , viscb  ,                                                     &
-  cpro_permeability_6 ,                                                 &
-  weighf , weighb ,                                                     &
+( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
+  iphydr , iwarnp ,                                                                  &
+  epsrgp , climgp , extrap ,                                                         &
+  rvoid  ,                                                                           &
+  cvar_pr   ,                                                                        &
+  coefa_p  , coefb_p  ,                                                              &
+  coefaf_p , coefbf_p ,                                                              &
+  viscf  , viscb  ,                                                                  &
+  cpro_permeability_6 ,                                                              &
+  weighf , weighb ,                                                                  &
   rhs   )
 
 endif
@@ -489,16 +491,16 @@ do while ( (isweep.le.nswmpr.and.residu.gt.epsrsm(ipr)*rnormp) &
 
     call itrgrv &
     !==========
-  ( init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
-    iphydr , iwarnp ,                                                     &
-    epsrgp , climgp , extrap ,                                            &
-    rvoid  ,                                                              &
-    cvar_pr   ,                                                           &
-    coefa_p  , coefb_p  ,                                                 &
-    coefaf_p , coefbf_p ,                                                 &
-    viscf  , viscb  ,                                                     &
-    cpro_permeability_6 ,                                                 &
-    weighf , weighb ,                                                     &
+    ( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
+    iphydr , iwarnp ,                                                                    &
+    epsrgp , climgp , extrap ,                                                           &
+    rvoid  ,                                                                             &
+    cvar_pr   ,                                                                          &
+    coefa_p  , coefb_p  ,                                                                &
+    coefaf_p , coefbf_p ,                                                                &
+    viscf  , viscb  ,                                                                    &
+    cpro_permeability_6 ,                                                                &
+    weighf , weighb ,                                                                    &
     rhs   )
 
   endif
@@ -604,15 +606,15 @@ else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrmav &
   !==========
- ( init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
-   iphydr , iwarnp ,                                              &
-   epsrgp , climgp , extrap ,                                     &
-   rvoid  ,                                                       &
-   presa  ,                                                       &
-   coefa_p , coefb_p , coefaf_p , coefbf_p ,                      &
-   viscf  , viscb  ,                                              &
-   cpro_permeability_6 ,                                          &
-   weighf , weighb ,                                              &
+ ( f_id0, init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
+   iphydr , iwarnp ,                                                     &
+   epsrgp , climgp , extrap ,                                            &
+   rvoid  ,                                                              &
+   presa  ,                                                              &
+   coefa_p , coefb_p , coefaf_p , coefbf_p ,                             &
+   viscf  , viscb  ,                                                     &
+   cpro_permeability_6 ,                                                 &
+   weighf , weighb ,                                                     &
    imasfl , bmasfl )
 
   ! The last increment is not reconstructed to fullfill exactly the continuity
@@ -625,15 +627,15 @@ else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrmav &
   !==========
- ( init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,  &
-   iphydr , iwarnp ,                                               &
-   epsrgp , climgp , extrap ,                                      &
-   rvoid  ,                                                        &
-   dpvar  ,                                                        &
-   coefa_p , coefb_p , coefaf_p , coefbf_p ,                       &
-   viscf  , viscb  ,                                               &
-   cpro_permeability_6 ,                                           &
-   weighf , weighb ,                                               &
+ ( f_id0, init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,  &
+   iphydr , iwarnp ,                                                      &
+   epsrgp , climgp , extrap ,                                             &
+   rvoid  ,                                                               &
+   dpvar  ,                                                               &
+   coefa_p , coefb_p , coefaf_p , coefbf_p ,                              &
+   viscf  , viscb  ,                                                      &
+   cpro_permeability_6 ,                                                  &
+   weighf , weighb ,                                                      &
    imasfl , bmasfl )
 
 endif
